%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all
close all
clc

myRain_date = fileread('../Table_climate_out.csv');
tmp = textscan(myRain_date, '%s %f %f %f %f %f %f %f', 'Delimiter', ';','HeaderLines',1, 'EmptyValue', Inf);
myRain_date = num2str(cell2mat(tmp{1}));
myRain_date = string(myRain_date);
myRain_date = datetime(myRain_date,'InputFormat','dd.MM.yyyy HH:mm:ss', 'Format','yyyy-MM-dd HH:mm');

myRain = readmatrix('../Table_climate_out.csv', 'NumHeaderLines', 1, 'Delimiter', ';');

% solar global radiation from Battipaglia
QGRout = myRain(:,7);

% % C Air temperature outside the greenhouse
Tout = myRain(:,2);
% % % Relative humidity outside the  greenhouse
RHout = myRain(:,3);

% Wm-2 â ¦Câ  1 k-Value Heat transmission coefficient of glazing
gr_k = 0.2;
% % Transmittance of the glazing material
tauC = 70 / 100;


% m length
gr_len = 30;
% m width
gr_wid = 10;
% m gutter high
gr_gut = 4;
% m ridge high
gr_rid = 6.3;
% m Average greenhouse height
H = gr_rid / 2;
HE = H; % ET is divided by H.
% m2 Area of the greenhouse floor surface
Afl = 300;
% m2 Area of the glazing surface
Agl = 674;
% m3 volume
gr_vol = gr_len * (gr_wid*gr_gut + 0.5*(gr_rid-gr_gut)*gr_wid);
% Ratio of glazing surface to floor surface
w = round(Agl/Afl, 1); % round as in paper

%%
%%%
% 0.003 to 0.172

% (m3 m-2 s-1) Ventilation rate
myqV = 0.00; % related to type of greenhouse and air exchanges per hour N
% % h-1 ventilation rate
% N = 120 % 2 ro 120

qvmax = 0.05;

% number of heaters
NH = 0;

% (-) Heater capacity
Hcap = 0; % 0, 75, 150 kW

% shades
Shade_perc = 0; % 0, 30 to 70

% (kgm-2 d-1) cooling system
EC = 0; % 0 7.4 or 14.8

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% parameters
% m Soil depth of layer 0
Z0 = 0.05;

% m Soil depth of layer 1
Z1 = 0.10;

% % Soil surface absorptivity
alpha = 70 / 100;

% Specific heat of moist air
Cp = 1013; % J / kg / °C % Allen 1996 FAO

% Heat capacity soil
Cs = 2000; % kJ / m3 / °C

% % Emissivity of air Layer, % so add / 100
epsilona = 75 / 100;

% % Soil surface emissivity, % so add / 100
epsilonf = 95 / 100;

% kJ m 2  C 1 h 1 Heat transfer coefficient at soil surface
hs = 25.2 / 3600; % hours to seconds

% kJm 1  C 1 h 1 Thermal conductivity of the soil
Ks = 5.5 / 3600; % hours to seconds

% % C Constant temperature at boundary layer, at 0.15m
% Tbl = unknown

% J kg-1 Latent heat of vaporization of water
L = 2.5e6;

% kg dry air m-3 Specific mass of air
rho = 1.2;

% Reflectance of the solar radiation on the ground
rhog = 0;
rhogTf = 0.5;

% Wm-2 K-4 Stefan-Boltzmann Constant
s_SB = 5.67e-8;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% controls
Tin_th = 20; %  °C

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Rv is the specific gas constant for water vapor defined as 461.5 J / kg / K
Rv = 461.5;
Rgas = 8.314; % J mol K
Vol = 1; % m3

% density air, linear approximation
T1air = 0; % C
T2air = 25; % C
r1air = 1.2922; % kg/m3
r2air = 1.1839; % kg/m3

% gwater kg-1   dry air Absolute humidity outside the greenhouse
% https://www.omnicalculator.com/physics/absolute-humidity
% https://www.youtube.com/watch?v=EXjbjIgTgsA
rair = r1air + (r2air - r1air) / (T2air - T1air) .* (Tout - T1air); % to have result in g/kg
Wout = 0.611*exp((17.502*Tout)./(Tout+240.97)).*RHout/100 * 18.02 / Rgas ./ (Tout + 273.15) * 1000 ./ rair; % result in g/kg

% % https://carnotcycle.wordpress.com/2012/08/04/how-to-convert-relative-humidity-to-absolute-humidity/
% Wout = 6.112 * exp( (17.62 * Tout) / (243.12 + Tout) ) * (RHout / 100) * 18.02 / (Rgas/100) / ToutK / rair

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Wm-2 Global radiation absorbed inside the greenhouse
QGRin = QGRout * tauC * (1-rhog);

% Wm-2 Heat flux from heating system
QHeater = NH * Hcap / Afl;

% T soil BC
TBL = 15; % Celsius

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% compute trajectory
h = 60; % s Time step in the numerical solution
tspan = 0:h:3600*size(myRain,1);
tspan = tspan(1:end-1)'; % end before beginning of new day
tspan_date = datetime( tspan/86400 + datenum(myRain_date(1)), 'TimeZone','UTC', 'ConvertFrom', 'datenum');
tspan_date.Format = 'yyyy-MM-dd HH:mm';


%%
%
% Tin C Air temperature inside the greenhouse
Tin = [];
% Win gwater kg-1   dry air Absolute humidity inside the greenhouse
Win = [];
% Tf C Ground surface temperature inside the greenhouse
Tf = [];
% test
dHsa = [];

% initial condition
Tin(1) = Tout(1);
Win(1) = Wout(1);
Tf(1) = TBL;
dHsa(1) = 0;

% other necessary variable within loop
RHin = [];
RHin(1) = RHout(1);
ET = []; % evapotranspiration
ET(1) = 0;

% from id rows dataframe to hourly values to minutes
qgrin = interp1(0:3600:3600*(size(myRain,1)-1),QGRin,tspan);
qgrin(isnan(qgrin)) = QGRin(end);
%
tout = interp1(0:3600:3600*(size(myRain,1)-1),Tout,tspan);
tout(isnan(tout)) = Tout(end);
%
wout = interp1(0:3600:3600*(size(myRain,1)-1),Wout,tspan);
wout(isnan(wout)) = Wout(end);
% wind speed from Battipaglia
myWiout = myRain(:,8);
myWout = interp1(0:3600:3600*(size(myRain,1)-1),myWiout,tspan);
myWout(isnan(myWout )) = myWiout(end);
myWout_copy = myWout;
% tmp = as.numeric(as.POSIXct(strptime("2024-04-30 00:00:00", format="%Y-%m-%d %H:%M:%S",tz="UTC")))
% tmp = seq(tmp , tmp + 24*3600, by=60)
% issue with 1 hour...
% tmp = seq(as.numeric(as.POSIXct(strptime(as.character(myRain$Date(1)), format="%Y-%m-%d %H:%M:%S",tz="UTC"))),
%            as.numeric(as.POSIXct(strptime(as.character(myRain$Date[nrow(myRain)]), format="%Y-%m-%d %H:%M:%S",tz="UTC"))),
%            by=60)
% tmp = tmp[-length(tmp)]
% tmp_date = data.frame(as.character(format(as.POSIXct(tmp - min(tmp), origin="1970-01-01", tz="UTC"), format="%Y-%m-%d %H:%M")))
% setdiff(x$Date , tspan)

% myh = ["00","01","02","03","04","05","06","21","22","23"]; % night
% tmp = char(tspan_date);
% iN = find(ismember(string(tmp(:,12:13)) , myh));
% EC = repelem(0,length(tspan));
% EC(iN) = 0; %1e-5 % cooling
EC = zeros(length(tspan),1);

iET = 1:60:length(tspan);
icontmax = 10;
toll = 1e-4; % for ET

conth = 1; % counter of hours, for selecting crop coefficient
prog = 0;
fprintf(1,'Computation Progress: %3d%%\n',prog);
%%
for ii=2:length(tspan)
    prog = ( 100*(ii/length(tspan)) );
    fprintf(1,'\b\b\b\b%3.0f%%',prog); % Deleting 4 characters (The three digits and the % symbol)


    % E
    % EC = 1e-5 % cooling
    ET(ii) = ET(ii-1);
    E = ET(ii) + EC(ii);
    % E = 0

    if (Tin(ii-1) > Tin_th)
        qV = qvmax;
    else
        qV = myqV;
        myWout(ii) = 0.1; % wind speed as ventilation rate m/s
    end
    if ismember(ii , iET)
        conth = conth + 1;
        vexit = 0;
        icont = 1;
        while (vexit == 0)

            if (icont == icontmax)
                vexit = 1;
                % print(paste0(ii, "th row did not converge. setting ET to current max"), sep="")
                ET(ii) = ET0_df;
            else
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % temperature step 1, Celsius to Kelvin because Cp
                xk0 = [qgrin(ii-1), Tin(ii-1) + 273.15, tout(ii-1) + 273.15];
                xk = xk0;

                % kelvin because Cp, result in K
                f1 = 1/(Cp * rho * H) * ( - (xk(2) - xk(3)) * (qV * Cp * rho) );
                xk = xk0 +(h/2)*f1;
                f2 = 1/(Cp * rho * H) * ( - (xk(2) - xk(3)) * (qV * Cp * rho) );
                xk = xk0 +(h/2)*f2;
                f3 = 1/(Cp * rho * H) * ( - (xk(2) - xk(3)) * (qV * Cp * rho) );
                xk = xk0 +h*f3;
                f4 = 1/(Cp * rho * H) * ( - (xk(2) - xk(3)) * (qV * Cp * rho) );

                d1a = (h/6)*(f1+2*f2+2*f3+f4);
                % xout1 = xk0 + d1

                % celsius because k, result in K
                xk0 = [qgrin(ii-1), Tin(ii-1), tout(ii-1)];
                xk = xk0;
                f1 = 1/(Cp * rho * H) * ( - (xk(2) - xk(3)) * (w * gr_k) );
                xk = xk0 +(h/2)*f1;
                f2 = 1/(Cp * rho * H) * ( - (xk(2) - xk(3)) * (w * gr_k) );
                xk = xk0 +(h/2)*f2;
                f3 = 1/(Cp * rho * H) * ( - (xk(2) - xk(3)) * (w * gr_k) );
                xk = xk0 +h*f3;
                f4 = 1/(Cp * rho * H) * ( - (xk(2) - xk(3)) * (w * gr_k) );

                d1b = (h/6)*(f1+2*f2+2*f3+f4);
                % xout1 = xk0 + d1

                % temperature step 2, result in K because Cp
                xk0 = [qgrin(ii-1), Tin(ii-1), tout(ii-1)];
                xk = xk0;

                %
                f1 = 1/(Cp * rho * H) * ( xk(1) + QHeater - L*E );
                xk = xk0 +(h/2)*f1;
                f2 = 1/(Cp * rho * H) * ( xk(1) + QHeater - L*E );
                xk = xk0 +(h/2)*f2;
                f3 = 1/(Cp * rho * H) * ( xk(1) + QHeater - L*E );
                xk = xk0 +h*f3;
                f4 = 1/(Cp * rho * H) * ( xk(1) + QHeater - L*E );

                d2 = (h/6)*(f1+2*f2+2*f3+f4);
                % xout2 = xk0 + d2

                % transfer heat soil air. assume to use previous time step for simplicity
                % 1000 is to convert W to kJ
                % this is in Celsius because Cs
                dHsa(ii) =   1/(Cs * Z0) * ( hs * (Tin(ii-1) - Tf(ii-1)) + epsilonf * s_SB / 1000 * epsilona * ( (273.15+Tin(ii-1))^4 - (273.15+Tf(ii-1))^4 ) );

                % because derivatives in K are the same derivatives in C, we sum derivatives disregarding units
                nowtin = xk0(2) + d1b + d2 - dHsa(ii);

                % if Tin becomes larger or lower than Tout because of ventilation only, then control because it calls air from outside
                if ( (d1a > 0 && nowtin < tout(ii) && (nowtin + d1a) > tout(ii)) || (d1a < 0 && nowtin > tout(ii) && (nowtin + d1a) < tout(ii)) )
                    Tin(ii) = tout(ii);
                else
                    Tin(ii) = nowtin + d1a;
                end

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % RH step 1
                xk0 = [Win(ii-1), wout(ii-1)];
                xk = xk0;

                f1 = 1/(H * rho) * ( - (xk(1) - xk(2)) * qV * rho) / 1;
                xk = xk0 +(h/2)*f1;
                f2 = 1/(H * rho) * ( - (xk(1) - xk(2)) * qV * rho) / 1;
                xk = xk0 +(h/2)*f2;
                f3 = 1/(H * rho) * ( - (xk(1) - xk(2)) * qV * rho) / 1;
                xk = xk0 +h*f3;
                f4 = 1/(H * rho) * ( - (xk(1) - xk(2)) * qV * rho) / 1;

                d1 = (h/6)*(f1+2*f2+2*f3+f4);
                % xout1 = xk0 + d1

                % RH step 2
                xk0 = [Win(ii-1), wout(ii-1)];
                xk = xk0;

                f1 = 1/(HE * rho) * E * 1000; % ET Penman-M is kg/m2/s, but for dimensionality E must be in g
                xk = xk0 +(h/2)*f1;
                f2 = 1/(HE * rho) * E * 1000; % ET Penman-M is kg/m2/s, but for dimensionality E must be in g
                xk = xk0 +(h/2)*f2;
                f3 = 1/(HE * rho) * E * 1000; % ET Penman-M is kg/m2/s, but for dimensionality E must be in g
                xk = xk0 +h*f3;
                f4 = 1/(HE * rho) * E * 1000; % ET Penman-M is kg/m2/s, but for dimensionality E must be in g

                d2 = (h/6)*(f1+2*f2+2*f3+f4);
                % d2 = 2 % test
                % xout2 = xk0 + d2

                % approach 2 use RH in the calculations which changes based on T
                Win(ii) = xk0(1) + d1 + d2;

                % approach to regulate RH at high qV
                if (Win(ii) < wout(ii))
                    Win(ii) = wout(ii);
                end

                rair = r1air + (r2air - r1air) / (T2air - T1air) * (Tin(ii) - T1air); % g/m3 to g/kg
                RHin(ii) = Win(ii) * 100 / (0.611*exp((17.502*Tin(ii))/(Tin(ii)+240.97))) * Rgas * (Tin(ii)+273.15) / 18.02 * rair / 1000; % with absolute humidity in g/kg

                if (RHin(ii) > 100)
                    RHin(ii) = 100;
                    % magnus formula for vapour pressure
                    Win(ii) = 0.611*exp((17.502*Tin(ii))/(Tin(ii)+240.97)) * RHin(ii)/100 * 18.02 / Rgas / (Tin(ii)+273.15) * 1000 / rair; % rsult in g/kg
                end

                % alternative simplistic id cannot regulate RH at high qV
                % rair = r1air + (r2air - r1air) / (T2air - T1air) * (Tin(ii) - T1air) % g/m3 to g/kg
                % RHin(ii) = wout(ii) * 100 / (0.611*exp((17.502*Tin(ii))/(Tin(ii)+240.97))) * Rgas * (Tin(ii)+273.15) / 18.02 * rair / 1000 % with absolute humidity in g/kg

                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % temperature soil
                xk0 = [qgrin(ii-1)];
                xk = xk0;

                mydepth = Z0 + Z1;
                % mydepth = 1
                f1 = 1/(Cs * Z0) * ( rhogTf * alpha * xk(1) / 1000 + Ks * (TBL - Tf(ii-1)) * 2 / (mydepth)) + dHsa(ii);
                xk = xk0 +(h/2)*f1;
                f2 = 1/(Cs * Z0) * ( rhogTf * alpha * xk(1) / 1000 + Ks * (TBL - Tf(ii-1)) * 2 / (mydepth)) + dHsa(ii);
                xk = xk0 +(h/2)*f2;
                f3 = 1/(Cs * Z0) * ( rhogTf * alpha * xk(1) / 1000 + Ks * (TBL - Tf(ii-1)) * 2 / (mydepth)) + dHsa(ii);
                xk = xk0 +h*f3;
                f4 = 1/(Cs * Z0) * ( rhogTf * alpha * xk(1) / 1000 + Ks * (TBL - Tf(ii-1)) * 2 / (mydepth)) + dHsa(ii);

                d = (h/6)*(f1+2*f2+2*f3+f4);
                Tf(ii) = Tf(ii-1) + d;
                
                % calculate ET with penman-monteith
                ET0 = fao_Snyder_greenhouse_DLC_R_Rain_source_Fernand(Tin(ii),RHin(ii),myWout(ii),qgrin(ii),tspan_date(ii));

                ET0_df = ET0 / 3600; % from hour to seconds
                if (isempty(ET0_df) | isnan(ET0_df))
                    ET0_df = 0;
                    vexit = 1;
                    %print(paste0(ii, "th row has ET = NA, at ", tspan_date(ii), sep=""))
                else
                    ET0_df = ET0_df * 1; % crop coefficient - myKc(conth)
                    if ( abs(ET0_df - ET(ii)) <= toll )
                        vexit = 1;
                        ET(ii) = ET0_df;
                    else
                        E = ET0_df + EC(ii);
                        icont = icont + 1;
                    end
                end
            end
        end

    else

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % temperature step 1, Celsius to Kelvin because Cp
        xk0 = [qgrin(ii-1), Tin(ii-1) + 273.15, tout(ii-1) + 273.15];
        xk = xk0;

        % kelvin because Cp, result in K
        f1 = 1/(Cp * rho * H) * ( - (xk(2) - xk(3)) * (qV * Cp * rho) );
        xk = xk0 +(h/2)*f1;
        f2 = 1/(Cp * rho * H) * ( - (xk(2) - xk(3)) * (qV * Cp * rho) );
        xk = xk0 +(h/2)*f2;
        f3 = 1/(Cp * rho * H) * ( - (xk(2) - xk(3)) * (qV * Cp * rho) );
        xk = xk0 +h*f3;
        f4 = 1/(Cp * rho * H) * ( - (xk(2) - xk(3)) * (qV * Cp * rho) );

        d1a = (h/6)*(f1+2*f2+2*f3+f4);
        % xout1 = xk0 + d1

        % celsius because k, result in K
        xk0 = [qgrin(ii-1), Tin(ii-1), tout(ii-1)];
        xk = xk0;
        f1 = 1/(Cp * rho * H) * ( - (xk(2) - xk(3)) * (w * gr_k) );
        xk = xk0 +(h/2)*f1;
        f2 = 1/(Cp * rho * H) * ( - (xk(2) - xk(3)) * (w * gr_k) );
        xk = xk0 +(h/2)*f2;
        f3 = 1/(Cp * rho * H) * ( - (xk(2) - xk(3)) * (w * gr_k) );
        xk = xk0 +h*f3;
        f4 = 1/(Cp * rho * H) * ( - (xk(2) - xk(3)) * (w * gr_k) );

        d1b = (h/6)*(f1+2*f2+2*f3+f4);
        % xout1 = xk0 + d1

        % temperature step 2, result in K because Cp
        xk0 = [qgrin(ii-1), Tin(ii-1), tout(ii-1)];
        xk = xk0;

        %
        f1 = 1/(Cp * rho * H) * ( xk(1) + QHeater - L*E );
        xk = xk0 +(h/2)*f1;
        f2 = 1/(Cp * rho * H) * ( xk(1) + QHeater - L*E );
        xk = xk0 +(h/2)*f2;
        f3 = 1/(Cp * rho * H) * ( xk(1) + QHeater - L*E );
        xk = xk0 +h*f3;
        f4 = 1/(Cp * rho * H) * ( xk(1) + QHeater - L*E );

        d2 = (h/6)*(f1+2*f2+2*f3+f4);
        % xout2 = xk0 + d2

        % transfer heat soil air. assume to use previous time step for simplicity
        % 1000 is to convert W to kJ
        % this is in Celsius because Cs
        dHsa(ii) =   1/(Cs * Z0) * ( hs * (Tin(ii-1) - Tf(ii-1)) + epsilonf * s_SB / 1000 * epsilona * ( (273.15+Tin(ii-1))^4 - (273.15+Tf(ii-1))^4 ) );

        % because derivatives in K are the same derivatives in C, we sum derivatives disregarding units
        nowtin = xk0(2) + d1b + d2 - dHsa(ii);

        % if Tin becomes larger or lower than Tout because of ventilation only, then control because it calls air from outside
        if ( (d1a > 0 && nowtin < tout(ii) && (nowtin + d1a) > tout(ii)) || (d1a < 0 && nowtin > tout(ii) && (nowtin + d1a) < tout(ii)) )
            Tin(ii) = tout(ii);
        else
            Tin(ii) = nowtin + d1a;
        end
        % Tin(ii) = nowtin + d1a

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % RH step 1
        xk0 = [Win(ii-1), wout(ii-1)];
        xk = xk0;

        f1 = 1/(H * rho) * ( - (xk(1) - xk(2)) * qV * rho) / 1;
        xk = xk0 +(h/2)*f1;
        f2 = 1/(H * rho) * ( - (xk(1) - xk(2)) * qV * rho) / 1;
        xk = xk0 +(h/2)*f2;
        f3 = 1/(H * rho) * ( - (xk(1) - xk(2)) * qV * rho) / 1;
        xk = xk0 +h*f3;
        f4 = 1/(H * rho) * ( - (xk(1) - xk(2)) * qV * rho) / 1;

        d1 = (h/6)*(f1+2*f2+2*f3+f4);
        % xout1 = xk0 + d1

        % RH step 2
        xk0 = [Win(ii-1), wout(ii-1)];
        xk = xk0;

        f1 = 1/(HE * rho) * E * 1000; % ET Penman-M is kg/m2/s, but for dimensionality E must be in g
        xk = xk0 +(h/2)*f1;
        f2 = 1/(HE * rho) * E * 1000; % ET Penman-M is kg/m2/s, but for dimensionality E must be in g
        xk = xk0 +(h/2)*f2;
        f3 = 1/(HE * rho) * E * 1000; % ET Penman-M is kg/m2/s, but for dimensionality E must be in g
        xk = xk0 +h*f3;
        f4 = 1/(HE * rho) * E * 1000; % ET Penman-M is kg/m2/s, but for dimensionality E must be in g

        d2 = (h/6)*(f1+2*f2+2*f3+f4);
        % d2 = 2 % test
        % xout2 = xk0 + d2

        % approach 2 use RH in the calculations which changes based on T
        Win(ii) = xk0(1) + d1 + d2;

        % approach to regulate RH at high qV
        if (Win(ii) < wout(ii))
            Win(ii) = wout(ii);
        end

        rair = r1air + (r2air - r1air) / (T2air - T1air) * (Tin(ii) - T1air); % g/m3 to g/kg
        RHin(ii) = Win(ii) * 100 / (0.611*exp((17.502*Tin(ii))/(Tin(ii)+240.97))) * Rgas * (Tin(ii)+273.15) / 18.02 * rair / 1000; % with absolute humidity in g/kg

        if (RHin(ii) > 100)
            RHin(ii) = 100;
            % magnus formula for vapour pressure
            Win(ii) = 0.611*exp((17.502*Tin(ii))/(Tin(ii)+240.97)) * RHin(ii)/100 * 18.02 / Rgas / (Tin(ii)+273.15) * 1000 / rair; % rsult in g/kg
        end

        % rair = r1air + (r2air - r1air) / (T2air - T1air) * (Tin(ii) - T1air) % g/m3 to g/kg
        % RHin(ii) = wout(ii) * 100 / (0.611*exp((17.502*Tin(ii))/(Tin(ii)+240.97))) * Rgas * (Tin(ii)+273.15) / 18.02 * rair / 1000 % with absolute humidity in g/kg

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % temperature soil
        xk0 = [qgrin(ii-1)];
        xk = xk0;

        mydepth = Z0 + Z1;
        % mydepth = 1
        f1 = 1/(Cs * Z0) * ( rhogTf * alpha * xk(1) / 1000 + Ks * (TBL - Tf(ii-1)) * 2 / (mydepth)) + dHsa(ii);
        xk = xk0 +(h/2)*f1;
        f2 = 1/(Cs * Z0) * ( rhogTf * alpha * xk(1) / 1000 + Ks * (TBL - Tf(ii-1)) * 2 / (mydepth)) + dHsa(ii);
        xk = xk0 +(h/2)*f2;
        f3 = 1/(Cs * Z0) * ( rhogTf * alpha * xk(1) / 1000 + Ks * (TBL - Tf(ii-1)) * 2 / (mydepth)) + dHsa(ii);
        xk = xk0 +h*f3;
        f4 = 1/(Cs * Z0) * ( rhogTf * alpha * xk(1) / 1000 + Ks * (TBL - Tf(ii-1)) * 2 / (mydepth)) + dHsa(ii);

        d = (h/6)*(f1+2*f2+2*f3+f4);
        Tf(ii) = Tf(ii-1) + d;

        %
        ET(ii) = ET(ii-1);


    end
end

% resampling minute data RK4, but calculations are in seconds
ET_h = reshape(ET,h,size(myRain,1));
ET_h = sum(ET_h,1)*60;

Tin_h = mean(reshape(Tin,h,size(myRain,1)),1);
RHin_h = mean(reshape(RHin,h,size(myRain,1)),1);
Tf_h = mean(reshape(Tf,h,size(myRain,1)),1);

% plot(tspan_date(1:h:end),cumsum(ET_h))
% plot(tspan_date(1:h:end),Tin_h)
% plot(tspan_date(1:h:end),RHin_h)
% plot(tspan_date(1:h:end),Tf_h)


save('ET_h_Fernand',"myRain_date","ET_h","RHin_h","Tin_h","Tf_h","tspan_date");
% quit;


%%
h = 60;
load ET_h_Fernand.mat
Tin_h_Fern = Tin_h;
RHin_h_Fern = RHin_h;
Tf_h_Fern = Tf_h;
ET_h_Fern = ET_h;


%%
myRain_valid = readmatrix('../Table_climate_in.csv', 'NumHeaderLines', 1, 'Delimiter', ';');
Tin_m = myRain_valid(:,2);
RHin_m = myRain_valid(:,3);


S = Tin_h_Fern';
O = Tin_m;
RMSE=(sum((S-O).^2, 'omitnan')/max(size(S)))^0.5;
r=min(min(corr(S,O, "rows","complete")));
a=std(S,"omitnan")/std(O);
b=mean(S,"omitnan")/mean(O);
KGE=1-sqrt((r-1)^2+(a-1)^2+(b-1)^2);

fprintf('r^2: %4.2f\n', r^2);
fprintf('RMSE: %4.2f\n', RMSE);
fprintf('KGE: %4.2f\n', KGE);

S = RHin_h_Fern';
O = RHin_m;
RMSE=(sum((S-O).^2, 'omitnan')/max(size(S)))^0.5;
r=min(min(corr(S,O, "rows","complete")));
a=std(S,"omitnan")/std(O);
b=mean(S,"omitnan")/mean(O);
KGE=1-sqrt((r-1)^2+(a-1)^2+(b-1)^2);

fprintf('r^2: %4.2f\n', r^2);
fprintf('RMSE: %4.2f\n', RMSE);
fprintf('KGE: %4.2f\n', KGE);
